#[repr(u8)]
#[derive(Debug, Default)]
pub enum Level {
    Off = 0,
    Error,
    Warn,
    #[default]
    Info,
    Help,
    Debug
}

const C_RESET: &'static str = "\x1B[0m";
const C_ERROR: &'static str = "\x1B[1;31m";
const C_WARN:  &'static str = "\x1B[1;33m";
const C_INFO:  &'static str = "\x1B[1;32m";
const C_DEBUG: &'static str = "\x1B[1;35m";
const C_HELP:  &'static str = "\x1B[1;36m";

pub fn _log(level: Level, str: &str) {
     match level {
        Level::Off => return,
        Level::Error => println!("{C_ERROR}error{C_RESET}: {str}"),
        Level::Warn  => println!("{C_WARN}warn{C_RESET}: {str}"),
        Level::Info  => println!("{C_INFO}info{C_RESET}: {str}"),
        Level::Help  => println!("{C_HELP}help{C_RESET}: {str}"),
        Level::Debug => println!("{C_DEBUG}debug{C_RESET}: {str}"),
     }
}

#[macro_use]
pub mod log {
    #[macro_export]
    macro_rules! error {
        ($($arg:tt)*) => {
            crate::logger::_log(crate::logger::Level::Error, &format!($($arg)*))
        };
    }
    #[macro_export]
    macro_rules! warn {
        ($($arg:tt)*) => {
            crate::logger::_log(crate::logger::Level::Warn, &format!($($arg)*))
        };
    }
    #[macro_export]
    macro_rules! info {
        ($($arg:tt)*) => {
            crate::logger::_log(crate::logger::Level::Info, &format!($($arg)*))
        };
    }
    #[macro_export]
    macro_rules! help {
        ($($arg:tt)*) => {
            crate::logger::_log(crate::logger::Level::Help, &format!($($arg)*))
        };
    }
    #[macro_export]
    macro_rules! debug {
        ($($arg:tt)*) => {
            crate::logger::_log(crate::logger::Level::Debug, &format!($($arg)*))
        };
    }
}
