use std::collections::HashMap;

use crate::tokeniser::tokentype::*;

use super::{typ::Type, Ast};

#[derive(Debug, Clone)]
pub enum Expr {
    // Comment(Comment),
    Group(Box<Expr>),
    UnOp {
        typ: Punctuation,
        right: Box<Expr>,
    }, 
    BinOp {
        typ: Punctuation,
        left: Box<Expr>,
        right: Box<Expr>,
    },
    Literal(super::literal::Literal),
    ArrayIndex {
        name: Box<Expr>,
        index: Box<Expr>,
    },
    Path(Path),
    Call {
        path: Box<Expr>,
        params: CallParams, // Expr ~ (, Expr)*
    },
    //MethodCall {
    //    var_name: Box<Expr>,
    //    method_name: Ident,
    //    params: CallParams,
    //},

    /// the left side only exists on the /.|->/ chain
    FieldAccess {
        left: Box<Option<Expr>>,
        right: Box<Expr>,
    },
    PtrFieldAccess {
        left: Box<Option<Expr>>,
        right: Box<Expr>,
    },
    ForLoop {
        init: Box<Ast>,
        test: Box<Expr>,
        on_loop: Box<Expr>,
        body: Block,
    },
    WhileLoop {
        test: Box<Expr>,
        body: Block,
    },
    InfLoop {
        body: Block,
    },
    If(IfExpr),
    Struct {
        path: Path,
        fields: HashMap<Ident, Expr>,
    },
    Return(Box<Option<Expr>>),
    Break,
    Continue,
    Cast {
        left: Box<Expr>,
        right: Box<Type>
    },
}

impl Expr {
    pub fn unwrap_path(&self) -> Path {
        let Expr::Path(p) = self else {panic!("Unwrapping")};
        p.clone()
    }
}



#[derive(Debug, Clone)]
pub struct CallParams(pub Vec<Expr>);

#[derive(Debug, Clone)]
pub struct Block(pub Vec<Ast>);

#[derive(Debug, Clone)]
pub struct Path(pub Vec<Ident>);


#[derive(Debug, Clone)]
pub struct IfExpr {
    pub test: Box<Expr>,
    pub body: Block,
    pub else_if: Option<IfBranchExpr>
}

#[derive(Debug, Clone)]
pub enum IfBranchExpr {
    ElseIf(Box<IfExpr>),
    Else(Block)
}

