use std::collections::HashMap;

use super::{expr::{Block, Expr}, typ::Type, Ident, TString};


#[derive(Debug, Clone)]
pub enum Statement {
    Fn {
        struct_name: Option<Ident>,
        name: Ident,
        params: Vec<(Ident, Type)>,
        ret_type: Option<Type>,
        qual_const: bool,
        qual_extern: Option<TString>, // abi
        body: Option<Block>, // If None then its a type declaration
    },
    TypeAlias {
        name: Ident,
        typ: Type,
    },
    Struct {
        name: Ident,
        fields: Vec<(Ident, Type)>,
    },
    Enum {
        name: Ident,
        fields: Vec<Ident>,
    },
    ConstVar {
        name: Ident,
        typ: Type,
        val: Expr
    },
    StaticVar {
        name: Ident,
        typ: Type,
        val: Expr,
    },
    Let {
        name: Ident,
        typ: Option<Type>,
        val: Option<Expr>,
    },
}
