use std::collections::HashMap;

use ast::{expr::Block, Ast, Program};

use crate::tokeniser::{Token, tokentype::*};

pub mod ast;
mod expr;
mod stat;
mod utils;
mod typ;

type Result<T> = anyhow::Result<T>;

pub fn parse_program(mut tokens: Vec<Token>) -> Result<Program> {
    let mut prog_body = Vec::new();

    while !tokens.is_empty() {
        if let Some(item) = parse_item(&mut tokens)? {
            prog_body.push(item);
        } else {
            break
        }
    }


    Ok(Program {
        ast: Block(prog_body),
        enums: HashMap::new(),
        functions: HashMap::new(),
        member_functions: HashMap::new(),
        types: HashMap::new(),
        structs: HashMap::new()
    })
}

fn parse_item(tokens: &mut Vec<Token>) -> Result<Option<Ast>> {
    if let Some(stat) = stat::parse_statement(tokens)? {
        return Ok(Some(Ast::Statement(stat)));
    }
    if let Some(expr) = expr::parse_expr(tokens, 0, true)? {
        return Ok(Some(Ast::Expr(expr)));
    }
    Ok(None)
}





